******************************************************************************
      subroutine BiDiag( nelem, lgPrintIt)
      implicit none
*solve the bi-diagonal matrix for ionization balance
      integer nelem
      include "yield.com"
      logical lgPrintIt
      include "itrdim.par"
      include "limelm.par"
      include "parray.com"
      include "IonRange.com"
      include "RecomRate.com"
      include "secmet.com"
      include "hv2ion.com"
      include "PhotRate.com"
      include "heating.com"
      include "IonFracs.com"
      include "h.com"
      include "workf.com"
      include "worksv.com"
      include "cmeion.com"
      include "ZoneCnt.com"
      include "chargt.com"
      include "hetmet.com"
      include "DestCrt.com"
      include "CharExc.com"
      include "nsShells.com"
      include "search.com"
      include "ElNames.com"
      include "CollIonRate.com"
      include "iondun.com"
      include "InCvgRsn.com"
      include "IonStage.com"
      include "IOqq.com"
*
*     local variables
      logical lgNegPop , lgDone
      integer  i , ns , nej , nelec , max , LoopBig
      real heat , abundOld
*
      double precision sum , csav(32) , ratio
*
      LoopBig = 0
      lgDone = .false.
      do while (LoopBig.lt.5 .and. (.not.lgDone) )
        do i=1,nelem
          destroy(nelem,i) = 0.
          create(nelem,i) = 0.
        end do
*
*       get total destruction rates
        do i=IonLow(nelem),IonHigh(nelem)-1
*         collisional ionization
          destroy(nelem,i) = destroy(nelem,i) + CollidRate(nelem,i,1)
*         >>chng 96 oct 9, following was clobbered by above, must maintain
*         contents of destroy for Auger effect
*         destroy(nelem,i) = CollidRate(nelem,i,1)
*         photoionization loop
          do ns=1,nsShells(nelem,i)
*         >>chng 96 oct 17 add destruction here
            destroy(nelem,i) = destroy(nelem,i) +
     1        PhotoRate(nelem,i,ns,1)
*           following is most number of electrons freed
            nelec = nyield(nelem,i,ns)
*           following loop is over number of electrons that come out of shell
*           with multiple electron ejection
            do nej=2,nelec
*             this is highest possible stage of ionization -
*             do not want to ignore ionization that go beyond this
*             >>chng 96 oct 10 put highest dest into highest hole
              max = min( i+nej-1, IonHigh(nelem)-1 )
*             if( xIonFracs(nelem,i+nej-1).gt.1e-30 ) then
              if( xIonFracs(nelem,max).gt.1e-30 ) then
*               i points to current stage of ionization
                ratio = dble(xIonFracs(nelem,i)) / 
     1          dble(xIonFracs(nelem,max))
              else
                ratio = 1.
              endif
*             yield here is fraction removing ion electrons
*             destroy(nelem,i+nej-1) = destroy(nelem,i+nej-1) +
*             >>chng 96 oct 10
              destroy(nelem,max) = destroy(nelem,max) +
     1        PhotoRate(nelem,i,ns,1) * vyield(nelem,i,ns,nej) * ratio
*             >>chng 96 Oct 9, had been below, change nelec to nej
*    1        PhotoRate(nelem,i,ns,1) * vyield(nelem,i,ns,nelec) * ratio
            end do
          end do
*         this is charge transfer destruction
*         >>chng 96 Oct 1, break out H ct totally
          destroy(nelem,i) = destroy(nelem,i) + CTHeavy(i,1) +
     1    HCharExcIon(i,nelem)*hii
        end do
*
*       recombination rates
        do i=IonLow(nelem),IonHigh(nelem)-1
*         recombination 
          create(nelem,i) = RecombinRate(nelem,i)
*         charge transfer recombination
*         >>chng 96 Oct 1, break out H ct totally
          create(nelem,i) = create(nelem,i) + CTHeavy(i,2) +
     1    HCharExcRec(i,nelem)*hi
        end do
*
*       option to print ionization and recombination arrays
*       parray flag set with "print array" command
        if( lgPrtArry .or. lgPrintIt) then
          write(qq,'(/1x,a4,1x,''I'',1p,16e8.1/7x,16e8.1)') 
     1    chElNames(nelem),(destroy(nelem,i),i=1,IonHigh(nelem)-1)
          write(qq,'(1x,a4,1x,''C'',1p,16e8.1/7x,16e8.1)') 
     1    chElNames(nelem),(CollidRate(nelem,i,1),i=1,IonHigh(nelem)-1)
          write(qq,'(1x,a4,1x,''R'',1p,16e8.1/7x,16e8.1)') 
     2    chElNames(nelem),(create(nelem,i),i=1,IonHigh(nelem)-1)
          write(qq,'(1x,''char I'',1p,16e8.1/7x,16e8.1)') 
     1    (HCharExcIon(i,nelem)*hii,i=1,IonHigh(nelem)-1)
          write(qq,'(1x,''char R'',1p,16e8.1/7x,16e8.1)')
     2    (HCharExcRec(i,nelem)*hi,i=1,IonHigh(nelem)-1)
        endif
*
*       invert and solve bidiagonal matrix
        csav(IonLow(nelem)) = 1.d0
        sum = 1.d0
*
        do i=IonLow(nelem)+1,IonHigh(nelem)
          csav(i) = csav(i-1) * destroy(nelem,i-1) / create(nelem,i-1)
          sum = sum + csav(i)
        end do
*
        sum = dble( xIonFracs(nelem,0) ) / sum
*
        do i=1,IonLow(nelem)-1
          heating(nelem,i) = 0.
          xIonFracs(nelem,i) = 0.
        end do
        do i=IonHigh(nelem)+1,nelem+1
          xIonFracs(nelem,i) = 0.
        end do
        do i=IonHigh(nelem),nelem
          heating(nelem,i) = 0.
        end do
*
*       this is a sanity check
        lgNegPop = .false.
*
        xIonFracs(nelem,IonHigh(nelem)) = csav(IonHigh(nelem)) * sum
        if( xIonFracs(nelem,IonHigh(nelem)).lt.0. ) lgNegPop = .true.
*
        lgDone = .true.
        do i=IonLow(nelem),IonHigh(nelem)-1
          if( lgSearch .and.xIonFracs(nelem,i).gt.0. ) then
*           >>chng 96 oct 27, sulphur and above oscillated, corrected by
*           using log mean during search phase
            abundOld = xIonFracs(nelem,i)
            xIonFracs(nelem,i) = 
     1      10.**( 0.5*(log10(xIonFracs(nelem,i))+log10(csav(i)*sum) ) )
*           >>chng 97 jul 29, set flag to redo if changes happen
            if( xIonFracs(nelem,i).gt.1e-15 ) then
              if( abs(abundOld/xIonFracs(nelem,i)-1.).gt.0.2 )
     1        lgDone = .false.
            endif
          else
            abundOld = xIonFracs(nelem,i)
            xIonFracs(nelem,i) = csav(i) * sum
*           >>chng 97 jul 30, set flag to redo if changes happen
            if( xIonFracs(nelem,i).gt.1e-10 ) then
              if( abs(abundOld/xIonFracs(nelem,i)-1.).gt.0.2 )
     1        lgDone = .false.
            endif
          endif
          if( xIonFracs(nelem,i).lt.0. ) lgNegPop = .true.
        end do
        if( lgPrtArry .or. lgPrintIt) then
          write(qq,'(1x,''AbundX'',1p,16e8.1/7x,16e8.1)')
     2    (xIonFracs(nelem,i),i=1,IonHigh(nelem))
        endif
        LoopBig = LoopBig + 1
      end do
*
      if(.not.lgDone) then
        lgIonDone = .false.
        chInCvgRsn(1:6) = 'BiDiag'
        chInCvgRsn(7:8) = '  '
*       this is list of numbers, 1 to limelm+1
        chInCvgRsn(9:10) = chIonStage(nelem)
      endif
*
*     we now have final solution for this element
*
      do i=IonLow(nelem),IonHigh(nelem)-1
        heat = 0.
        do ns=1,nsShells(nelem,i)
*         heating by various sub-shells
          heat = heat + PhotoRate(nelem,i,ns,2)
        end do
        heating(nelem,i) = xIonFracs(nelem,i) * heat
*       total metal photo heating for LINES
        hetmet = hetmet + heating(nelem,i)
*       HeavIonCool is cooling due to collisional ionization of heavy elements
*       CollidRate(nelem,i,2) is cooling, erg/s/atom, eval in CollidIonize
        HeavIonCool = HeavIonCool + xIonFracs(nelem,i) *
     1  CollidRate(nelem,i,2)
*
*       secondary ionization rate, 
        secmet = secmet + hv2ion(nelem,i) * xIonFracs(nelem,i)
*       LA excit rate, =0 if ionized
        smetla = smetla + hv2la(nelem,i) * xIonFracs(nelem,i)
      end do
*
*     this can`t possibly happen
      if( lgNegPop ) then
        write(qq,'('' Negative population found for abundance of '',
     1  ''ionization stage of element '',A4,'', ZONE='',I4)') 
     2  chElNames(nelem) ,nzone
        write(qq,'('' Populations were'',1p,12e9.1)')
     1  (xIonFracs(nelem,i),i=1,IonHigh(nelem))
        write(qq,'('' destroy vector ='',1p,12e9.1)')
     1  (destroy(nelem,i),i=1,IonHigh(nelem)-1)
        lgNegPop = .false.
        do i=1,IonHigh(nelem)-1
          if( destroy(nelem,i).lt.0. ) lgNegPop = .true.
        end do
*       print some extra stuff if destroy was negative
        if( lgNegPop ) then
          write(qq,'('' CTHeavy  vector ='',1p,12e9.1)')
     1    (CTHeavy(i,1),i=1,IonHigh(nelem)-1)
          write(qq,'('' HCharExcIon vtr='',1p,12e9.1)')
     1    (HCharExcIon(i,nelem),i=1,IonHigh(nelem)-1)
          write(qq,'('' CollidRate  vtr='',1p,12e9.1)')
     1    (CollidRate(nelem,i,1),i=1,IonHigh(nelem)-1)
*         photo rates per subshell
          write(qq,'('' photo rates per subshell, ion'')')
          do i=1,IonHigh(nelem)-1
            write(qq,'(i3,1p,8e9.1)') i,(PhotoRate(nelem,i,ns,1),
     1      ns=1,nsShells(nelem,i))
          end do
        endif
*       now check out creation vector
        write(qq,'('' create  vector ='',1p,12e9.1)')
     1  (create(nelem,i),i=1,IonHigh(nelem)-1)
        call negcon
        call ShowMe
        stop
      endif
*
*     estimate of work function of total gas
      do i=IonLow(nelem),IonHigh(nelem)-1
        workfz = workfz + worksv(i) * xIonFracs(nelem,i+1)
      end do
*
      end
