******************************************************************************
      subroutine conorm
      implicit none
*normalize continuum to proper intensity
      include "fr1ryd.par"
      include "itrdim.par"
      include "en1ryd.par"
      include "ncell.par"
      include "limspc.par"
      include "SpeedLgt.par"
      include "terp.com"
      include "rfield.com"
      include "bounds.com"
      include "spnorm.com"
      include "range.com"
      include "spec.com"
      include "sptype.com"
      include "phycon.com"
      include "radius.com"
      include "called.com"
      include "trace.com"
      include "cover.com"
      include "fourpi.com"
      include "bit32.com"
      include "IOqq.com"
*
*     local variables
      integer i , ip , ipoint
      real ar1 , diff , flx1,ffun1,f, qentrd , qintr,pentrd,pintr,
     1 flx2
*
      ar1 = log10( radius )
*
*     this is a sanity check, it can't happen
      do i=1,nspec
        if( chRSpec(i).eq.'UNKN' ) then
          write(qq,'('' UNKN spectral normalization cannot happen.'')')
          write(qq,'('' CONORM punts.'')')
          stop
        else if( chRSpec(i).ne.'SQCM'.and. chRSpec(i).ne.'4 PI' ) then
          write(qq,'('' chRSpec must be SQCM or 4 PI, and it was '',A4,
     1    ''.  This cannot happen.'')') chRSpec(i)
          write(qq,'('' CONORM punts.'')')
          stop
        endif
*
*       this sanity check makes sure that atlas.mod or werner.mod grids
*       are for the current version of the code
        if( chSpType(i).eq.'VOLK' ) then
          ip = ipoint( 9.5 )
          diff = abs(tnu(i,ip)-anu(ip)) / anu(ip)
          if( diff.gt.0.001 ) then
            write(qq,'(1p,2e10.2)') anu(ip) , tnu(i,ip)
            write(qq,'('' CONORM: The energy grid of the stellar '',
     1      ''atmosphere file does not agree with the grid in this '',
     2      ''version of the code.'')')
            write(qq,'('' A stellar atmosphere grid from an old '',
     2      ''version of the code is probably in place.'')')
            write(qq,'('' A grid for the current version of Cloudy '',
     1      ''must be generated and used.'')')
            write(qq,'('' This is done with the COMPILE STARS '',
     1      ''command.'')')
            write(qq,'('' Sorry.'')')
            stop
          endif
        endif
      end do
*
*     default is intensities, but luminosity of any would override
*     following two values are correct for intensities
      pirsq = 0.
      lgFourPi = .false.
*     check whether ANY luminosities are present
      do i=1,nspec
        if( chRSpec(i).eq.'4 PI' ) then
          pirsq = 1.0992099 + 2.*ar1
          lgFourPi = .true.
*         convert down to intensity
          totpow(i) = totpow(i) - pirsq
          if(lgTrace)write(qq,'('' CONORM converts continuum'',I3,
     1    '' from luminosity to intensity.'')') i
        endif
      end do
*
*     convert any ionization parameters to q(h)
      do i=1,nspec
        if( chSpNorm(i).eq.'IONI' ) then
          totpow(i) = totpow(i) + log10(hden) + log10(SpeedLight)
          chSpNorm(i) = 'Q(H)'
          if(lgTrace)write(qq,'('' CONORM converts continuum'',i3,
     1    '' from ionizat par to q(h).'')') i
        endif
      end do
*
      if( lgTrace ) then
        if( lgFourPi ) then
          write(qq,'('' Cloudy will predict lumin into 4pi'')')
        else
          write(qq,'('' Cloudy will do surface flux for lumin'')')
        endif
      endif
*
*     if intensity per unit area is predicted then geometric
*     covering factor must be unity
*     variable can also be set elsewhere
      if( .not.lgFourPi ) then
        covgeo = 1.
      endif
*
*     main loop to find continuum normalization starts here
      do i=1,nspec
        ipspec = i
        if( lgTrace ) write(qq,'('' CONORM continuum number'',I3,
     1  '' is shape '',A5,'' range is '',1P,2E11.2)') i,chSpType(i),
     2  range(i,1),range(i,2)
*
        if( chSpNorm(i).eq.'RATI' ) then
*         option to scale relative to previous continua
*         this must come first since otherwise may be too late
*         BUT ratio cannot be the first continuum source
          if( lgTrace ) then
            write(qq,'('' CONORM this is ratio to 1st con'')')
          endif
*         check that this is not the first continuum source, we must ratio
          if(i.le.1) then
            write(qq,'('' I cant form a ratio if continuum is first '',
     1      ''source.'')')
            stop
          endif
*
*         first find photon flux and Q of prevous continuum
          ipspec = ipspec - 1
          flx1 = ffun1( range(i,1) ) * spfac( ipspec ) * range(i,1)
*         check that previous continua were not zero where ratio is formed
          if(flx1.le.0. ) then
            write(qq,'('' Previous continua were zero where ratio is '',
     1      ''desired.'')')
            stop
          endif
*
*         return pointer to previous (correct) value, find F, Q
          ipspec = ipspec + 1
*         we want a continuum totpow as powerful, flx is now desired flx
          flx1 = flx1 * totpow(i)
*.        find flux of this new continuum at that point
          flx2 = ffun1( range(i,2) ) * range(i,2)
*         this is ratio of desired to actual
          spfac(i) = flx1 / flx2
          if( lgTrace ) then
            write(qq,'('' CONORM ratio will set scale fac to''
     1      ,1P,E10.3,'' at'',E10.2,'' Ryd.'')') totpow(i) , range(i,1)
          endif
*
        else if( chSpNorm(i).eq.'FLUX' ) then
*         specify flux density
*         option to use arbitrary frequency or range
          f = log10( max(1e-37,ffun1( range(i,1)) ) ) + 
     1    log10( range(i,1)*en1ryd/fr1ryd )
          f = ( totpow(i) - f )
*     >>chng 96 dec 31, added following test
          if( lgBit32 ) then
            if( f.gt.35. ) then
              write(qq,'('' Continuum source'',i3,'' is too intense'',
     1        '' for this cpu - is it normalized correctly?'')')i
              stop
            endif
          endif
          spfac(i) = 10.** f
          if( lgTrace ) then
            write(qq,'('' CONORM will set log fnu to'',1P,E10.3,
     1      '' at'',E10.2,'' Ryd.  Factor='',E11.4)')
     2      totpow(i) , range(i,1) , spfac(i)
          end if
*
        else if( chSpNorm(i).eq.'Q(H)' .or. chSpNorm(i).eq.'PHI ' ) then
*         some type of photon density entered
          qentrd = qintr( range(i,1) , range(i,2) )
          diff = (totpow(i) - qentrd )
          if( lgBit32.and.(diff.lt.-25. .or. diff.gt.35.) ) then
            write(qq,'('' Continuum source specified is too extreme'',
     1      '' for a 32 bit cps.'')')
            stop
          else
            spfac(i) = 10.**diff
          endif
          if(lgTrace) write(qq,'('' CONORM finds Q over '',
     1    ''range from'',1p,e11.4,''-'',E11.4,'' Ryd, Factor='',
     2    e11.4)')
     4    range(i,1) , range(i,2) , spfac(i)
*
        else if( chSpNorm(i).eq.'LUMI' ) then
*         luminosity entered, special since default is TOTAL lumin
          pentrd = pintr( range(i,1) , range(i,2) ) + log10(en1ryd)
          f = totpow(i) - pentrd
*     >>chng 96 dec 31, added following test
          if( lgBit32 ) then
            if( f.gt.35. ) then
              write(qq,'('' Continuum source'',i3,'' is too intense'',
     1        '' for this cpu - is it normalized correctly?'')')i
              stop
            endif
          endif
          spfac(i) = 10.** f
          if(lgTrace) write(qq,'('' CONORM finds luminosity '',
     1    ''range is'',1P,E10.3,'' to '',E9.3,'' Ryd, factor is'',
     2    E11.4)')
     4    range(i,1) , range(i,2) , spfac(i)
        else
          write(qq,'('' What chSpNorm label is this?'',A4)')chSpNorm
          stop
        endif
*
*       sec part after .or. added June 93 because sometimes spfac=0
*       got past first test
        if( 1./spfac(i) .eq.0. .or.  spfac(i).eq.0. ) then
          write(qq,'('' CONORM finds infinite continuum scale '',
     1    ''factor.'')')
          write(qq,'('' The continuum is too intense to compute with''
     1    ,'' this cpu.'')')
          write(qq,'('' Were the intensity and luminosity commands'',
     1    ''switched??'')')
          write(qq,'('' Cloudy punts.  Sorry.'')')
          stop
        endif
      end do
*
      end
