******************************************************************************
      real function ffun1( xnu )
      implicit none
*derive flux at a specific energy, for one continuum
      real xnu
*
      include "te1ryd.par"
      include "ncell.par"
      include "limspc.par"
      include "evryd.par"
      include "rfield.com"
      include "cnread.com"
      include "terp.com"
      include "spec.com"
      include "bounds.com"
      include "IOqq.com"
      include "sptype.com"
      include "NumUpper.com"
      include "bit32.com"
*
      real small , big
      parameter (small=1e-25, big=1e10 )
      logical lgWarn
      save lgWarn
*
*     local variables
      character*5 chKey
      real sexp , fac , y
      integer i , j , ipoint
*
      data lgWarn/.false./
*
*     FFUN1 returns photons per unit time, area, energy, for one continuum
*     ipspec is the pointer to the continuum source, in the order
*     entered on the command lines
*
      chKey = chSpType(ipspec)
*
      if( chKey.eq.'AGN  ' ) then
*       power law with cutoff at both ends
*       nomenclature all screwed up - slope is cutoff energy in Ryd,
*       cutoff(i,1) is ratio of two continua from alpha ox
*       cutoff(i,2) is slope of bb temp
        ffun1 = xnu **(-1.+cutoff(ipspec,2) ) * sexp(xnu/slope(ipspec))
     1  * sexp(0.01/xnu)
*       only add on x-ray for energies above 0.1 Ryd
        if( xnu.gt.0.1 ) then
          if( xnu.lt.7350. ) then
*           cutoff is actually normalization constant
*           below 100keV continuum is nu-1
            ffun1 = ffun1 + xnu**(cutoff(ipspec,3)-1.)*cutoff(ipspec,1)*
     1      sexp(1./xnu)
          else
            ffun1 = ffun1 + 
     1      7350.**(cutoff(ipspec,3)-1.)*cutoff(ipspec,1)*
     2      (xnu/7350.)**(-3)
          endif
        endif
*
      else if( chKey.eq.'POWER' ) then
*       power law with cutoff at both ends
        ffun1 = xnu **(-1.+slope(ipspec)) *
     1  sexp(  xnu/cutoff(ipspec,1)  +  cutoff(ipspec,2)/xnu  )
*
      else if( chKey.eq.'BLACK' ) then
*       black body
        fac = te1ryd * xnu/slope(ipspec)
        if( fac.ge.80. ) then
          ffun1 = 0.
        else if( fac.lt.80. .and. fac.gt.0.01 )  then
          ffun1 = xnu*xnu/(exp(fac) -1.)
        else
          ffun1 = xnu*xnu / ( fac*(1.+fac/2.) )
        endif
*
      else if( chKey.eq.'INTER' ) then
*       interpolate on input spectrum
        if( xnu.ge.tnu(ipspec,1) ) then
          do i=2,ncell
            j = i - 1
            if( xnu .lt. tnu(ipspec,i) ) then
              y = tfac(ipspec,j) + tslop(ipspec,j) *
     1        log10( xnu/tnu(ipspec,j) )
              ffun1 = (10. ** y) / xnu
              return
            endif
          end do
*         energy above highest in table
          ffun1 = 0.
        else
*         energy below lowest on table
          ffun1 = 0.
        endif
*
      else if( chKey.eq.'BREMS' ) then
*       brems continuum, rough gaunt factor
        fac = te1ryd * xnu / slope(ipspec)
        ffun1 = sexp( fac ) / xnu**1.2
*
      else if( chKey.eq.'LASER' ) then
*       a laser, mostly one frequency
        if(xnu.gt.0.95*slope(ipspec) .and. xnu.lt.1.05*slope(ipspec))
     1  then
          ffun1 = big
        else
          ffun1 = small
        endif
*
      else if( chKey.eq.'READ ' ) then
*       if this is first time called then we need to read in file
        if( tslop(ipspec,1).eq.0. ) then
          call ReadTable( int(tslop(ipspec,2) ) )
          tslop(ipspec,1) = 1.
        endif
*       use array of values read in on TABLE READ command
        if( xnu.ge.egamry ) then
          ffun1 = 0.
        else
          i = ipoint( xnu )
          if( i.gt.nupper.or.i.lt.1 ) then
            ffun1 = 0.
          else
            ffun1 = ConTabRead(i) / anu(i) / anu(i)
          endif
        endif
*
      else if( chKey.eq.'VOLK ' ) then
*       use array of values read in from Kevin Volk's rebinning of
*       large atlas grids
        if( xnu.ge.egamry ) then
           ffun1 = 0.
        else
           i = ipoint( xnu )
           if( i.gt.ncell ) then
             write(qq,'('' Too many points - increase ncell'')')
             write(qq,'('' cell needed='',i4,'' ncell='',i4)')
     1       i , ncell
             stop
           endif
*          change nupper to upper limit of stellar continuum
           if( i.gt.nupper.or.i.lt.1 ) then
              ffun1 = 0.
           else
* bug fixed Jul 9 93: FFUN1 = TSLOP(IPSPEC,I) / ANU(I) / ANU(I)
*             i has value 939
              ffun1 = tslop(ipspec,i) / anu(i)
           endif
        endif
      else
        write(qq,'('' I do not understand continuum label '',A5)')
     1  chKey
        stop
      endif
*
      if( ffun1.gt.1e35 .and. (.not.lgWarn) .and. lgBit32 ) then
        lgWarn = .true.
        write(qq,'('' FFUN1:  Continuum'',i4,'' is very intense for'',
     1  '' a 32 bit cpu.'')') ipspec
        write(qq,'('' I will try to press on, but may have '',
     1  ''problems.'')')
      endif
*
      end
