******************************************************************************
      subroutine GetInter( chCard , lgEOF )
      implicit none
*parse parameters on interpolate command
      character chCard*80
      logical lgEOF
*
      include "ncell.par"
      include "limspc.par"
      include "called.com"
      include "bndsok.com"
      include "terp.com"
      include "spnorm.com"
      include "spec.com"
      include "sptype.com"
      include "bounds.com"
      include "egray.com"
      include "conbug.com"
      include "trace.com"
      include "bit32.com"
      include "ninput.com"
      include "IOqq.com"
*
*     local variables
      logical lgEOL , lgDONE
      integer i , j 
      real fread , cmax , cmin , fac
*
*     this sub reads in the "interpolate" command, and has
*     logic for the "continue" lines as well.
*     OUTPUT:
*     TNU is vector of eneriges where the grid is defined
*     TSLOP initially is vector of log fnu at each freq
*     converted into slopes here too
*
      nspec = nspec + 1
      if( nspec.ge.limspc) then
        write(qq,'('' Too many spectra entered.  Increase LIMSPC'')')
        stop
      endif
*
      chSpType(nspec) = 'INTER'
*
*     read all of the numbers on a line
      j = 0
*     this is flag saying that all numbers are in
      lgDONE = .false.
*     this flag says we hit end of command stream
      lgEOF = .false.
      do while( .not. lgDONE .and. .not.lgEOF)
        i = 5
        lgEOL = .false.
*       keep scanning numbers til we hit eol for current line image
        do while(.not.lgEOL)
          j = min( j+1 , ncell )
          tnu(nspec,j) = fread( chCard , i , 76 , lgEOL )
          tslop(nspec,j) = fread( chCard , i , 76 , lgEOL )
        end do
        j = j - 1
*
*       read a new line, checking for EOF
        call readar( chCard , lgEOF )
*       option to ignore all lines starting with #, *, or %.
        do while( .not.lgEOF .and. (
     1    chCard(1:1).eq.'#' .or. chCard(1:1).eq.'*' .or. 
     1    chCard(1:1).eq.'%' ) )
          call readar( chCard , lgEOF )
        end do
*
        call caps( chCard )
        if( chCard(1:4) .eq. 'CONT' ) then
          if(lgTalk) write(qq,'(23X,''*'',1X,A80,''*'')') chCard
        else
*         we have a line but next command, not continue
          lgDONE = .true.
        endif
*       this is another way to hit end of input stream - blank lines
        if( chCard(1:1) .eq. ' ' ) lgEOF = .true.
      end do
*
*     if valid next line, backup one line
      if( lgDONE ) then
        nRead = nRead - iReadWay
      endif
*
*     done reading all of the possible lines
      j = j - 1
      if( j.lt.1 ) then
        write(qq,'('' There must be at least 2 pairs to interpolate,'')
     1  ')
        stop
      else if( j.gt.ncell-2 ) then
        write(qq,'('' Too many continuum points entered.'')')
        stop
      endif
*
      if( tnu(nspec,1).eq.0. ) then
*       special case - if first energy is zero then it is low energy
        if( tnu(nspec,2).gt.0. ) then
*         second energy positive, assume linear Ryd
          tnu(nspec,1) = emm
        else if( tnu(nspec,2).lt.0. ) then
*         second energy negative, assume log of Ryd
          tnu(nspec,1) = log10( emm )
        else
*         second energy zero, not allowed
          write(qq,'('' An energy of zero was entered for element'',
     1    i3,'' in INTERPOLATE and is not allowed.  SORRY'')') i
          stop
        endif
      endif
*
*     convert from log(Hz) to Ryd if first nu>5
      if( tnu(nspec,1).ge.5. ) then
        do i=1,j+1
          tnu(nspec,i) = 10.** (tnu(nspec,i)-15.517)
        end do
      else if( tnu(nspec,1).lt.0. ) then
*       energies entered as logs
        do i=1,j+1
          tnu(nspec,i) = 10.**tnu(nspec,i)
        end do
      else
*       numbers are linear Rydbergs
        do i=1,j+1
          if( tnu(nspec,i).eq.0. ) then
            write(qq,'('' An energy of zero was entered for element'',
     1      I3,'' in INTERPOLATE and is not allowed.  SORRY'')') i
            stop
          end if
        end do
      endif
*
      do i=1,j
*       check that frequencies are monotonically increasing
        if( tnu(nspec,i+1).le.tnu(nspec,i) ) then
          write(qq,'('' The energies MUST be in increasing order.'',
     1    ''  Energy #'',I3,''='',1P,E10.2,'' Ryd was greater than'',
     2    '' or equal to the next one.  Sorry.'')') i , tnu(nspec,i)
          stop
        endif
*       TFAC is energy, and TSLOP is slope in f_nu; not photons
        tfac(nspec,i) = tslop(nspec,i)
        tslop(nspec,i) = (tslop(nspec,i+1)-tslop(nspec,i)) /
     1  log10(tnu(nspec,i+1) / tnu(nspec,i))
      end do
      tfac(nspec,j+1) = tslop(nspec,j+1)
*
      if( j+2.lt.ncell ) then
*       zero out remainder of array
        do i=j+2,ncell
          tnu(nspec,i) = 0.
        end do
      endif
*
*     now check that array is defined over all energies
      if( (tnu(nspec,1).gt.emm) ) then
*       not defined over low energy part of array
        write(qq,'('' Warning:  the input continuum was not defined '',
     1   ''over the entire energy range. Some energies are set to '',
     2  ''zero.''/'' >>>>>>>>>>You are making a big mistake.'')')
      endif
*
*     check on IR
      if( tnu(nspec,1).gt.emm )   lgMMok = .false.
      if( tnu(nspec,1).gt.1/36.) then
        lgHPhtOK = .false.
      endif
*     gamma ray, EGAMRY is roughly 100MeV
      if( tnu(nspec,j+1).lt.egamry ) lgGamrOK = .false.
*     EnerGammaRay is roughly 100keV; high is gamma ray
      if( tnu(nspec,j+1).lt.EnerGammaRay) lgXRayOK = .false.
*
*     find min and max of continuum
      cmax = -38.
      cmin = 28
*     tfac can be very large or small
      do i=1,j
        cmax = max( cmax , tfac(nspec,i) )
        cmin = min( cmin , tfac(nspec,i) )
      end do
*
*     check on dynamic range of input continuum
      if( cmax - cmin .gt.74. .and. lgBit32 ) then
        write(qq,'('' The dynamic range of the specified continuum'',
     1  ''is too large for a 32-bit computer.  Sorry.'')')
        stop
      endif
*
      if( cmin.lt.-37. ) then
        fac = -cmin - 37.
        do i=1,j
          tfac( nspec,i ) = tfac( nspec,i ) + fac
        end do
      else if( cmax.gt.37. ) then
        fac = cmax - 37.
        do i=1,j
          tfac( nspec,i ) = tfac( nspec,i ) - fac
        end do
      endif
*
*     option to print out results at this stage - "trace continuum"
      if( lgConBug.and.lgTrace ) then
        write(qq,'('' Table for this continuum; TNU, TFAC, TSLOP'')')
        write(qq,'(1p,3e12.4)')(tnu(nspec,i),tfac(nspec,i),
     1  tslop(nspec,i),i=1,j+1)
      endif
*
*     renormalize continuum so that quantity passes through unity
*     at 1 Ryd
*     >>chng 97 june 8, get rid of statement labels
*     find 1 ryd, j is one less than number of continuum pairs
      i = 1
      do while( tnu(nspec,i).lt.1. .and. i.lt.j )
        i = i + 1
      end do
*     j is 2 if continuum has 3 points (dqher.in tests this)
*     if j is 1 then i is 1, if j is 2 then i is 2
      if( j.gt.2 .and. tnu(nspec,i).lt.1. ) then
        write(qq,'('' TABLE fails sanity test in GetInter'')')
        stop
      endif
*
*     do the renormalization so 1 ryd is about unity
      fac = tfac(nspec,i)
      do i=1,j+1
        tfac(nspec,i) = tfac(nspec,i) - fac
      end do
*
      end
