************************************************************************
       subroutine he2cs123(i,j,temp,cs)
       implicit none
*line collision strengths for lower levels of helium ion, n=1,2,3, by K Korista
       real temp
       integer i,j
       real cs, cs3s, cs3p, cs3d, t, a, b, c
*
*      These are fits to Table 2
*      >>refer Aggarwal, K.M., Callaway, J., Kingston, A.E., Unnikrishnan, K.
*      >>refer 1992, ApJS, 80, 473
*      He II collision strengths for 1s-2s, 1s-2p, 1s-3, 2s-3, and 2p-3, 
*      principle quantum numbers n and l.
*      CLOUDY sums angular momentum (s,p,d) components of level n=3.
*
*      i is the lower level and runs from 1 to 3 (1s, 2s, 2p)
*      j is the upper level and runs from 2 to 4 (2s, 2p, 3)
*      check temperature: fits only good between 5,000K and 500,000K
*
       t = temp
       if(t .lt. 5000.) then
         t = 5000.
       else if(t .gt. 5.0e+05) then
         t = 5.0e+05
       endif

**************fits begin here**************

       if( i .eq. 1 .and. j .eq. 2 ) then
* 1s - 2s
         a = 0.12176209
         b = 0.039936166
         c = 143284.77

         cs = a + b*exp(-t/c)

       else if ( i .eq. 1 .and. j .eq. 3 ) then
* 1s - 2p
         a = 0.32916723
         b = 2.9711166e-05
         c = 0.73158545

         cs = a + b*t**c

       else if ( i .eq. 1 .and. j .eq. 4 ) then
* 1s - 3s
         a = 0.46546497
         b = -0.020835863
         c = -2.159172

         cs3s = a + b*alog(t) + c/alog(t)

* 1s - 3p
         a = 0.044501688
         b = 3.0508137e-04
         c = 0.43254802

         cs3p = a + b*t**c

* 1s - 3d
         a = 0.040523277
         b = -2.004485e-15
         c = 2.1338557

         cs3d =  a + b*t**c

         cs = cs3s + cs3p + cs3d

       else if (i .eq. 2 .and. j .eq. 4) then
* 2s - 3s
         a = 0.5234889
         b = 4.41475e-06
         c = 8.9899702e-06
         
         cs3s = (a + c*t)/(1 + b*t)

* 2s - 3p
         a = 1.4903214
         b = 1.0622666e-05
         c = -2.9001451e-12

         cs3p = a + b*t + c*t*t

* 2s - 3d
         a = 1.4215094
         b = 2.0538877e-06
         c = 1.762076e-05

         cs3d = (a + c*t)/(1 + b*t)

         cs = cs3s + cs3p + cs3d

       else if (i .eq. 3 .and. j .eq. 4) then
* 2p - 3s
         a = 1.0295881
         b = 0.80638448
         c = 52741.735

         cs3s = a + b*exp(-t/c)

* 2p - 3p
         a = 4.769306
         b = 2.0967075e-06
         c = -2153.1219

         cs3p = a + b*t + c/t

* 2p - 3d
         a = 9.7226127
         b = 7.6089851e-05
         c = -3.3996921e-11

         cs3d = a + b*t + c*t*t

         cs = cs3s + cs3p + cs3d

       else 

**************fits end here**************
         write(6,*) '  insane levels for He II n=1,2,3 !!!'
         stop
       endif
       end
