******************************************************************************
      real function HydColDwn(ihigh,ilow,iz)
      implicit none
*rate coefficient for collisional de-excitation for any levels of hydrogenic Z
*written by Jason Ferguson to compute the 
*     >>refer Sampson, D.H., & Zhang, H.L. 1998, ApJ, 335, 516
*     rate coeff. for collisional excitation between any two levels of 
*     hydrogenic atoms charge Z.  Valid for all ilow, ihigh.
*
      integer iz
      integer ihigh,ilow
      include "phycon.com"
      include "IOqq.com"
*
*     local variables
      real H,An,R,xp,xn,xnp2,tev,ryd,y,e1,e2,A,D,c,rate
      real arrH(4),arrAn(10),arrR(10)
      real ee1, fosc, sexp 
      integer i
      data (arrH(i),i=1,4)/1.48,3.64,5.93,8.32/
      data (arrAn(i),i=1,10)/1.30,0.59,0.38,0.286,
     1     0.229,0.192,0.164,0.141,0.121,0.105/
      data (arrR(i),i=1,10)/1.83,1.60,1.53,1.495,
     1     1.475,1.46,1.45,1.45,1.46,1.47/
*
*begin sanity check
      if( iz.lt.2 .or. ilow.ge.ihigh .or. ilow*ihigh.le.0 ) then
        write(qq,'(''HydColDwn called with impossible parameters.'')')
        write(qq,'(3i5)') ihigh,ilow,iz
        stop
      endif
*end sanity check
*
      if(ilow.gt.4) then
        H = 2.15 * ilow
      else
        H = arrH(ilow)
      endif
      if(ilow.gt.10) then
        An = 1./float(ilow)
        R = 1.50
      else
        An = arrAn(ilow)
        R = arrR(ilow)
      endif
*
      xp=float(ihigh)
      xn=float(ilow)
      xnp2 = (xn/xp)**2
      tev = 8.61716e-5 * te
      ryd = 13.60583
*
      y = iz**2 * ryd * (1./xn**2 - 1./xp**2) / tev
      if(y.lt.1.) then
        e1 = ee1(y)
        e2 = sexp(y) - y*e1
      else
        e1 = ee1(y)
        e2 = 1 - y*e1
      endif
      A = 4.*xn**4 / ( 1 - xnp2 ) * fosc(xn,xp)
      D = A * H * ( (1-xnp2)**R - An*xnp2 )
      c = 1.12 * xn * A * (1 - xnp2)
      if(xp-xn.eq.1.) then
        c = c * sexp( 0.006*(xn-1.)**6/iz )
      endif
*
      rate = 8.63e-6/( (xn*iz)**2 * sqrt(te) )
      rate = rate * ( (A + y*c)*e1 + D*e2 )
*
*     Convert from excitation to de-excitation
      if(y.lt.1) then
        HydColDwn = rate * xnp2 / sexp(y)
      else
        HydColDwn = rate * xnp2 
      endif
*
      HydColDwn = max( HydColDwn , 1.e-30 )
*
      return
      end
