****************************************************************
      subroutine hypho(zed,n,lmin,lmax,en,ncell,anu,bfnu)
      implicit none
*generate hydrogenic photoionization cross sections
      real zed, en
      integer n,lmin,lmax,ncell
      include "IOqq.com"
*     *************************************************************
*          GENERATE HYDROGENIC PHOTOIONIZATION CROSS SECTIONS
*
*     Hypho calculates Hydrogenic bound-free (BF) xsectns for each n,l .
*     For Opacity work we calculate l-weighted average for each n .
*     BF xsectns for Hydorgenic ion are tabulated at E/z**2. E is
*     the energy in Ry corresponding to the frequencies on the Opacity
*     mesh. It can changed accoding to need.
*
* zed=Z-Nelc, 
* n=principle quantum number
* lmin=lowest angular momentum considered of level n
* lmx=highest angular momentum considered of level n
* en=zed*zed/(n*n) lowest energy at which the hydrogenic cross sections are
* to be calculated
* anu = photon energy
* bfnu=photoionization cross section in cm^-2
*
*     local variables
      integer nlv, nsl, ntg, ncm , nfreq , i,j,il,if,k,l,
     1 lmax1,ilmax,lg,lin,m,muls,ll,lm,lll,llm,mulr,llk,
     1 mulp
      real zed2,fn,sn4,sn,gl,al,fth,g12,se,e,p,ee,exp1,gne,
     1 con2,ai,gn0,g11,a,gn1e,g22,g32,g21,g31,sl,sm,sm4,
     2 sum,p1,p2,alfac,flm,fll,gnt,sl4
      parameter (nlv=850,nsl=40,ntg=20,ncm=3000)
      real bfnu(ncm) , anu(ncm)
      parameter (nfreq = ncm )
      real g2(nfreq,2),g3(nfreq,2),ab(nfreq),alo(7000),
     cfal(7000),freq(nfreq)
      save g2,g3,ab,alo, fal,freq
      integer mm(nfreq)
      double precision fac , x , con3 , anl
      logical first
* CON1=conversion factor from a0**2 to Mb, x-sectns in megabarns
*     CON1=8.5594e-19 *  1.e+18
      real zero,one,two,four,ten,con1
      data first /.true./
      data zero/0./ , one/1./ , two/2./,four/4./,ten/10./,
     1 con1/0.8559/
*
      if( ncell.gt.ncm ) then
        write(qq,'('' increase ncm in hypho to at least'',i5)') ncell
        stop
      endif
*
      if( first ) then
        fal(1) = zero
        do i=2,7000
          ai = float(i - 1)
          alo(i-1) = log10(ai)
          fal(i) = alo(i-1) + fal(i-1)
        end do
        first = .false.
      endif
C
C          CALCULATION OF HYDROGENIC PHOTOIONIZATION CROSS SECTION
C
C         INITIALIZATION FOR N
C * con2=(a0**2*1.e+18)*(n/zed)**2, zed=z-nelc
c * for hydrogen, the threshold, fth=1/n**2, and for hydrogenic atom,it is
c zed**2/n**2.
              zed2 = zed * zed
      fn =float(n)
      sn = fn*fn
      sn4 = four*sn
      con2=con1*(fn/zed)**2
      fth=one/sn
c
      gn0=2.3052328943  -2.302585093  *fal(n+n)-fn*0.61370563888  +
     .alo(n)*(fn+one)*2.30258093
      lmax1=min(lmax,n-1)
      ilmax=n-lmin
c * calculate top-up st.wt for given n and lmin=lmax+1 (R-matrix). subtract
c the sum from 2n**2.
      gl = 0.
      if (lmin .ne. 0) then
        do i = 1, lmin
          lg = i - 1
          gl = gl + 2. * lg + 1
        end do
      endif
      gnt = 2. * ( n*n - gl )
C define photon energies epnu corresponding to hydrogenic atom with charge
c z and freq to hydrogen atom;  INITIALIZE G'S
      do i=1,ncell
        mm(i) = 0
        bfnu(i) = zero
        freq(i)=anu(i)*zed2
        do j=1,2
          g2(i,j) = zero
          g3(i,j) = zero
        end do
      end do
*     gjf Aug 95 change
*     original code returned total &(*(*$# if freq(1)<en
      freq(1) = max( freq(1) , en )
c
C calculate cross section:  L LOOP
c
       do 21 il = 1, ilmax
           l = n - il
           lin=l-lmin+1
           m=0
           al = float(l)
           k = n - l - 2
           con3=con2/(two*al+one)
c
C          FREQUENCY LOOP (FREQ UNITS ARE RYD*ZED**2)
C
           do 51 if = 1, ncell
             if(freq(if).lt.fth) then
                if(l.le.lmax1) anl=0.
                go to 51
             endif
              g11 = zero
              g12 = zero
                m=m+1
                se=freq(if)-fth
            if (se.lt.0.) write(8,'(1x,i4,1p2e12.4)')if,freq(if),fth
c            if ( se .lt. -1.e-30) se = 1.e-06
                e=sqrt(se)
                x=one+sn*se
            if ( k .lt. 0 ) then
                     if (e.ge.0.314  ) then
                          ee=6.2831853  /e
                          p=0.5  *alog10(exp1(-ee))
                     else
                          p = zero
                     endif
                     if(e.gt.1.0E-6) then
                          a=two*(fn-atan(fn*e)/e)
                     else
                          a=zero
                     endif
                     ab(m)=(gn0+a)/2.302585  -p-(fn+two)*log10(x)
                     gne = 0.1
                     gn1e = x*gne/(fn + fn)
                     g3(m,2) = gne
                     g3(m,1) = gn1e
                     g2(m,2) = gne*fn*x*(fn+fn-one)
                     g2(m,1) = gn1e*(fn+fn-one)*(four+(fn-one)*x)
            endif
                g22 = g2(m,2)
                g32 = g3(m,2)
                g21 = g2(m,1)
                g31 = g3(m,1)
                muls = mm(m)
                if(k)20,30,25
C
C                     l.lt.n-2
C
25              if(k.gt.1) go to 28
                ll = n - 1
                lm = n - 2
28              sl = float(ll*ll)
                sl4 = four*sl
                fll = float(ll)
                g12 = (sn4 - sl4 + (two*sl - fll)*x)*g22 -
     .          sn4*(sn - sl)*(one + (fll + one)**2*se)*g32
                if (l.eq.0) go to 12
                sm = float(lm*lm)
                sm4 = four*sm
                flm = float(lm)
                g11 = (sn4 - sm4 + (two*sm + flm)*x)*g21 -
     .          sn4*(sn - (flm + one)**2)*(one + sm*se)*g31
                g31 = g21
                g21 = g11
12              g32 = g22
                g22 = g12
c
                    if ( m .ne. ncell ) go to 31
                         ll = ll - 1
                    if ( l .eq. 0 ) go to 31
                         lm = lm - 1
  31                continue
c
                if (g12.lt.1.E20) go to 40
                muls = muls + 35
                g22 = g22*1.E-35
                g32 = g32*1.E-35
                g12 = g12*1.E-35
                if (l.eq.0) go to 40
                g11 = g11*1.E-35
                g21 = g21*1.E-35
                g31 = g31*1.E-35
                go to 40
C
c                    l.eq.n-1
c
20              g11 = g31
                if (l.eq.0) g11 = zero
                g12 = g32
                go to 40
C
C                    l.eq.n-2
C
30              g11 = g21
                if (l.eq.0) g11 = zero
                g12 = g22
40              continue
                mm(m) = muls
                g2(m,2) = g22
                g3(m,2) = g32
                g2(m,1) = g21
                g3(m,1) = g31
                alfac = fal(n+l+1) - fal(n-l) + two*(al-fn)*alo(2*n)
                p1 = one
                lll = l + 1
                llm = l - 1
                mulr = 0
                if (llm.lt.1) go to 46
                do  1985 i = 1,llm
                     ai = float(i)
                     p1 = p1*(one + ai*ai*se)
                     if (p1.ge.1.E20) then
                          p1 = p1*1.E-10
                          mulr = mulr + 10
                     endif
 1985            continue
46              p2 = p1
                llk = llm + 1
                if (llk.lt.1) llk = 1
                do i = llk,lll
                     ai = float(i)
                     p2 = p2*(one + ai*ai*se)
                end do
                mulp = 0
17             if (g12.lt.one) go to 117
               mulp = mulp - 10
               g12 = g12*1.E-10
               if (l.eq.0) go to 17
               g11 = g11*1.E-10
               go to 17
117            sum=alfac+float(mulr)+two*(ab(m)+float(muls-mulp+1))
               fac = zero
*     >>chng 96apr19, 35 had been 50
               if(abs(sum).lt.35.  ) fac=ten**sum
               if (l.eq.0) go to 18
               g11 = g11*p1*g11
18             g12 = g12*p2*g12
C               anl=con3*(g11 *al + g12 *(al+one))
C    .         *x*fac
               if(l.le.lmax1)anl=fac*x*con3*(g11*al+g12*(al+1.))
                    anl = anl * 2. * (2.* al + 1. )
c
           bfnu(if) = bfnu(if) + anl*1e-18
c
  51        continue
c
  21       continue
c
c              bfmin = 1.e-03 * bfnu(1) / gnt
            do 61 i = 1, ncell
  61        bfnu(i) = bfnu(i) / gnt
c
      end
