******************************************************************************
      subroutine level2( tarray )
      implicit none
*do level population and cooling for two level atom
      include "nta.par"
      real tarray(nta)
*
      include "ncell.par"
      include "limelm.par"
      include "ipLnArry.par"
      include "limLevelN.par"
      include "nwindline.par"
      include "dcool.com"
      include "IonFracs.com"
      include "p2ovp1.com"
      include "phycon.com"
      include "fslev.com"
      include "edsqte.com"
      include "PopLevls.com"
      include "WindEtc.com"
      include "corind.com"
      include "IOqq.com"
*
*     local variables
      character chLab*4
      character chIonLbl*4
      real g1 , g2 , a21 , boltz , AbunxIon , omega , sexp ,
     1 col21 , col12 , rate12 , r , plower,coolng,aem,
     2 ri21 , Enr12 , Enr21
      integer nelem , ion , ipCont
*
*     result is density (cm-3) of excited state times A21
*     result normalized to N1+N2=ABUND
*     routine increments dCooldT, call coladd
*     CDSQTE is EDEN / SQRTE * 8.629E-6
*
      if( ipWEtc.gt.0 ) then
        ion = ipWindIon(ipWEtc)
        nelem = ipWindNelem(ipWEtc)
        chLab = chWinLab(ipWEtc)
      else
        ion = tarray(ipLnIonStg) + 0.00001
        nelem = tarray(ipLnNelem) + 0.00001
        chLab = chIonLbl( tarray )
      endif
*
*     net rate down A21*(escape + destruction)
      a21 = tarray(ipLnAul) * ( tarray(ipLnEscP)+tarray(ipLnDesP) )
*
*     continuum pointer
      ipCont =  int(tarray(ipLnIpCont) )
*
*     approximate boltzmann factor to see if results zero
      boltz = ContBoltz( ipCont )
*
*     xIonFracs(nelem,i) is density of ith ionization stage (cm^-3)
      AbunxIon = xIonFracs( nelem , ion )
*
*     ROUGH check whether upper level has significant population
      r = (boltz*cdsqte + tarray(ipLnPump))/(cdsqte+a21)
*
*     following first test needed for 32 bit cpu on search phase
*     >>chng 96 july 2, was 1e-30 crashed on dec, change to 1e-25
*     if( AbunxIon.lt.1e-25 .or. boltz.gt.30. ) then
*     >>chng 96 july 11, to below, since can be strong pumping when
*     boltzman factor essentially zero
      if( AbunxIon.lt.1e-25 .or. r.lt.1e-25 ) then
*       put in pop since possible just too cool
        tarray(ipLnPopl) = AbunxIon
        tarray(ipLnPopOpc) = AbunxIon
        tarray(ipLnPopu) = 0.
        tarray(ipLnInten) = 0.
        tarray(ipLnCool) = 0.
        tarray(ipLnNPhots) = 0.
        tarray(ipLnOTS) = 0.
        tarray(ipLnAovTOT) = 1.
        tarray(ipLnColovTOT) = 0.
        tarray(ipLnHeat) = 0.
*       level populations
        PopLevels(0) = AbunxIon
        PopLevels(1) = 0.
        DepLTELevels(0) = 1.
        DepLTELevels(1) = 0.
*       this is used for line radiation pressure
        p2ovp1 = 0.
*       add zero to cooling
        call coladd( chLab(1:4) , int(tarray(ipLnWlAng)),0.)
*       now add zero to ots field
*       this is high-quality line if .eq.0
        if( tarray(ipLnCS1) .eq.0. ) 
     1  call AddOTSLin( 0. , ipCont ,1.0)
        return
      endif
*
*     statistical weights of upper and lower levels
      g1 = tarray(ipLnGl)
      g2 = tarray(ipLnGu)
* 
*     now get real Boltzmann factor
      boltz = tarray(ipLnBolt) / te
*begin sanity checks
      if( boltz .le.0. ) then
        write(qq,'('' level2 called with non-positive Boltzmann'',
     1  '' factor, ='',1P,E10.2)') boltz
        write(qq,'('' wl='',1P,E10.2)') tarray(ipLnWlAng)
        write(qq,'('' line label='',a10)') chLab
        stop
      endif
*end sanity checks
      boltz = sexp(boltz)
*
*     collision strength for this transition
      omega = tarray(ipLnCS)
*
*begin sanity checks
      if(g1.le.0. .or. g2.le.0. ) then
        write(qq,'('' level2 called with non-positive statistical'',
     1  '' weights, ='',1P,2E10.2)') g1,g2
        write(qq,'('' wl='',1P,E10.2)') tarray(ipLnWlAng)
        write(qq,'('' label='',a4)')chLab
      endif
      if( omega.le.0. ) then
        write(qq,'('' level2 called with non-positive collision'',
     1  '' strength, ='',1P,E10.2)') omega
        write(qq,'('' wl='',1P,E10.2)') tarray(ipLnWlAng)
        write(qq,'('' line label='',a10)') chLab
        stop
      endif
*end sanity checks
*
      col21 = cdsqte * omega
*     rate 1 to 2 is both collisions and pumping
      col12 = col21 / g1 * boltz
      col21 = col21 / g2
*
      rate12 = col12 + tarray(ipLnPump)
*     induced emissions down
      ri21 = tarray(ipLnPump) * g1/g2
      r = (a21+col21+ri21) / rate12
*
*     upper level pop
      pfs2 = AbunxIon / (r+1.)
      PopLevels(1) = pfs2
      tarray(ipLnPopu) = pfs2
*     pop of ground
*     PopLevels(0) = AbunxIon - PopLevels(1)
*
*     lower level population
      pfs1 = pfs2 * r
      PopLevels(0) = pfs1
*
*     departure coef of excited state rel to ground
      DepLTELevels(0) = 1.
      if( boltz.gt.1e-20.and.PopLevels(1).gt.1e-20 ) then
*       this line could have zero boltz factor but radiatively excited
*       dec alpha does not obey () in fast mode??
        DepLTELevels(1) =
     1   ( PopLevels(1)/PopLevels(0) )  /
     2   ( boltz * g2/g1 )
      else
        DepLTELevels(1) = 0.
      endif
*
*     number of escaping line photons, used elsewhere for outward beam
      tarray(ipLnNPhots) = tarray(ipLnAul) * tarray(ipLnEscP) *pfs2
*
*     intensity of line
      tarray(ipLnInten) = tarray(ipLnNPhots)*tarray(ipLnEnrErg)
      plower = AbunxIon - pfs2
*
*     ratio of collisional to total excitation
      tarray(ipLnColovTOT) = col12/rate12
*
*     two cases - collisionally excited (usual case) or 
*     radiatively excited - in which case line can be a heat source
*     following are correct heat exchange, if will mix to get correct deriv
      Enr12 = plower*col12 *tarray(ipLnEnrErg)
      Enr21 = pfs2*col21*tarray(ipLnEnrErg)
*     energy exchange due to this level
*     net cooling due to excit minus part of de-excit
      tarray(ipLnCool) = Enr12 - Enr21*tarray(ipLnColovTOT)
*     net heating is remainder
      tarray(ipLnHeat) = Enr21*(1.-tarray(ipLnColovTOT))
      coolng = tarray(ipLnCool)
*
*     expression pre jul 3 95, changed for case where line heating dominates
*     coolng = (plower*col12 - pfs2*col21)*tarray(ipLnEnrErg)
*     tarray(ipLnCool) = coolng
*
*     add to cooling - heating part was taken out above,
*     and is not added in here - it will be added to heating(1,23)
*     in SumCool
      call coladd( chLab(1:4) , int(tarray(ipLnWlAng)) , coolng )
*
*     derivative of cooling function
      dCooldT = dCooldT + coolng*( tarray(ipLnBolt)*tsq1 - halfte )
*
*     destroyed part of line
      tarray(ipLnOTS) = pfs2 * tarray(ipLnAul) * tarray(ipLnDesP)
*
      if( tarray(ipLnOTS).lt.0. ) then
        write(qq,'('' level2 finds negative ots rate, rate and wl are'',
     1  1P,E10.2,0P,F7.0)') tarray(ipLnOTS),tarray(5)
        write(qq,'(1P,10E10.2)') tarray(17),a21,col21,col12,pfs2,
     1  plower,coolng
        write(qq,'(1p,10e10.2)') tarray(1),tarray(11),tarray(13),
     1  tarray(18)
        stop
      endif
*
*     add this line to ots field
      if( tarray(ipLnCS1) .eq.0. ) 
     1  call AddOTSLin( tarray(ipLnOTS) , ipCont ,0.5)
*
*     compute ratio Aul/(Aul+Cul)
      aem = tarray(ipLnAul) * tarray(ipLnEscP)
      tarray(ipLnAovTOT) = aem / (aem+col21+ri21)
*
*     following is relative popultation per sub-level, used
*     to calculate radiation pressure in TEMP
      p2ovp1 = g1 / ( g2*r )
*     level population with correction for stimulated emission
      tarray(ipLnPopl) = PopLevels(0) 
      tarray(ipLnPopOpc) = PopLevels(0) - PopLevels(1)*g1/g2
*
      end
