      subroutine phfit(nz,ne,is,e,s,lghs)
      implicit none
*derive photoionization cross sectoins for first 30 elements
      integer nz,ne,is
      real e,s
      logical lghs
*** Version 3. October 8, 1996.
*** Written by D. A. Verner, verner@pa.uky.edu
*** Inner-shell ionization energies of some low-ionized species are slightly
*** improved to fit smoothly the experimental inner-shell ionization energies 
*** of neutral atoms.
*** New logical argument lghs
******************************************************************************
*** This subroutine calculates partial photoionization cross sections
*** for all ionization stages of all atoms from H to Zn (Z=30) by use of
*** the following fit parameters:
*** Outer shells of the Opacity Project (OP) elements:
***    Verner, Ferland, Korista, Yakovlev, 1996, ApJ, in press.
*** Inner shells of all elements, and outer shells of the non-OP elements:
***    Verner and Yakovlev, 1995, A&AS, 109, 125
*** Input parameters:  nz - atomic number from 1 to 30 (integer) 
***                    ne - number of electrons from 1 to iz (integer)
***                    is - shell number (integer)
***                    e - photon energy, eV 
***                    lghs - logical, "FALSE" (default) calculates 
***                           new cross sections, "TRUE" calculates
***                           only old Hartree-Slater cross sections
*** Output parameter:  s - photoionization cross section, Mb
*** Shell numbers:
*** 1 - 1s, 2 - 2s, 3 - 2p, 4 - 3s, 5 - 3p, 6 - 3d, 7 - 4s. 
*** If a species in the ground state has no electrons on the given shell,
*** the subroutine returns s=0.
******************************************************************************
      integer L
      COMMON/L/L(7)
      integer NINN
      COMMON/NINN/NINN(30)
      integer NTOT
      COMMON/NTOT/NTOT(30)
      real PH1 , PH2
      COMMON/PH1/PH1(6,30,30,7)
      COMMON/PH2/PH2(7,30,30)
*
*     local variables
      integer nout  , nint
      real einn,p1,y,q,a,b,x,z
*
      s=0.0
      if(nz.lt.1.or.nz.gt.30)return
      if(ne.lt.1.or.ne.gt.nz)return
      nout=NTOT(ne)
      if(nz.eq.ne.and.nz.gt.18)nout=7
      if(nz.eq.(ne+1).and.(nz.eq.20.or.nz.eq.21.or.nz.eq.22.or.nz.
     &eq.25.or.nz.eq.26))nout=7
      if(is.gt.nout)return
      if(is.eq.6.and.(nz.eq.20.or.nz.eq.19).and.ne.ge.19)return
      if(e.lt.PH1(1,nz,ne,is))return
      nint=NINN(ne)
      if(nz.eq.15.or.nz.eq.17.or.nz.eq.19.or.
     &(nz.gt.20.and.nz.ne.26))then
         einn=0.0
      else
         if(ne.lt.3)then
            einn=1.0e+30
         else
            einn=PH1(1,nz,ne,nint)
         endif
      endif
      if(is.le.nint.or.e.ge.einn.or.lghs)then
         p1=-PH1(5,nz,ne,is)
         y=e/PH1(2,nz,ne,is)
         q=-0.5*p1-L(is)-5.5
         a=PH1(3,nz,ne,is)*((y-1.0)**2+PH1(6,nz,ne,is)**2)
         b=sqrt(y/PH1(4,nz,ne,is))+1.0
         s=a*y**q*b**p1
      else
         if(is.lt.nout.and.is.gt.nint.and.e.lt.einn)return
         p1=-PH2(4,nz,ne)
         q=-0.5*p1-5.5
         x=e/PH2(1,nz,ne)-PH2(6,nz,ne)
         z=sqrt(x*x+PH2(7,nz,ne)**2)
         a=PH2(2,nz,ne)*((x-1.0)**2+PH2(5,nz,ne)**2)
         b=1.0+sqrt(z/PH2(3,nz,ne))
         s=a*z**q*b**p1
      endif
      return
      end
