******************************************************************************
      subroutine ReadTable(io)
      implicit none
*called by TABLE READ to read in continuum from PUNCH TRANSMITTED CONTINUUM
      integer io
      include "ncell.par"
      include "limpun.par"
      include "rydlam.par"
      include "evryd.par"
      include "rfield.com"
      include "tmn.com"
      include "cnread.com"
      include "pnunit.com"
      include "IOqq.com"
*
*     local variables
      character chExtra*80
      integer ierr , i , nPoints
      real Differ , fread , EnerLast
      logical lgEOL
*
*     check whether any other punch on this unit
      do i=1,npunch
        if( io.eq.ipPnunit(i) ) then
          write(qq,'('' ReadTable: PUNCH output has been requested '',
     1    ''on the same i/o unit as TABLE READ - continuum file has '',
     2    ''been clobbered.'')')
          write(qq,'('' The conflicting unit is'',i4)') io
          write(qq,'('' Please confirm no punch/read conflicts, '',
     1    ''recreate continuum file, and rerun.'')')
          stop
        endif
      end do
*
*     ok if called iteratively, since closed at the end
      open(io,iostat=i,status='old')
      if( i.ne.0 ) then
        write(qq,'('' TABLE READ did not find fort file, unit='',i3)')io
        stop
      endif
*
*     read in first two lines of header
      read(io,'(a80)') chExtra
*     first number on this line is version number, confirm that
*     this is greater than 90.01 due to bug in punch tran con in 90.00 
      i = 1
*     first number should be version number of code
      Differ = fread( chExtra , i , 76 , lgEOL )
      if( Differ.lt.90.04 ) then
        write(qq,'(a80)') chExtra
        write(qq,'('' This transmitted continuum file was produced '',
     1  ''by an older version of Cloudy.  '')')
        write(qq,'('' It must be recreated with version 90.04 or '',
     1  ''later.'')')
        write(qq,'('' The version I read in was'',f6.2)') Differ
        write(qq,'('' Sorry.'')')
        stop
      endif
      read(io,'(a80)') chExtra
*
*     now read in the file of numbers
      ierr = 0
      i = 1
      do while( ierr.eq.0 .and. i.lt.ncell )
        read(io,*,iostat=ierr) tmn(i) , ConTabRead(i)
        i = i + 1
      end do
*     put pointer at last good value
      nPoints = i - 2
*
*     check that sane number of values entered
      if( nPoints.lt.1 ) then
        write(qq,'('' ReadTable, fort file for TABLE READ has too few'',
     1  '' points, ='',i5)') nPoints
        stop
      endif
*
*     check on units of energy scale, convert to Rydbergs if necessary
*     tmn is scale read in from punch file, anu is correct scale
*     EnerLast is energy of last point in rydbergs
      EnerLast = tmn(nPoints)
      if( abs(tmn(1) - anu(1)) / anu(1) .gt.1e-3 ) then
*       first energy does not appear to have been in Rydbergs
        if ( tmn(1).lt.0. ) then
          write(qq,'('' ReadTable:  the first energy in table read '',
     1    ''file is not positive.  Something is wrong.'')')
          stop
        else if ( abs(tmn(1) - rydlam/anu(1)*1e-4)/tmn(1).lt.1e-3 ) then
*         wavelength in microns
          EnerLast = rydlam / tmn(nPoints) /1e4
        else if ( abs(tmn(1) - rydlam/anu(1))/tmn(1).lt.1e-3 ) then
*         wavelength in Angstroms
          EnerLast = rydlam / tmn(nPoints) 
        else if ( abs(tmn(1) - anu(1)*evryd*1e-3)/tmn(1).lt.1e-3 ) then
*         wavelength in keV
          EnerLast = tmn(nPoints) / evryd / 1e-3
        else if ( abs(tmn(1) - anu(1)*evryd)/tmn(1).lt.1e-3 ) then
*         wavelength in eV
          EnerLast = tmn(nPoints)/evryd
        else
          write(qq,'('' ReadTable:  the energy scale in the table '',
     1    ''read file makes no sense to me.'')')
          stop
        endif
      endif
*
*     now check now the energies of the highest points agree
      Differ = abs( EnerLast - anu(nPoints) ) / anu(nPoints)
      if( Differ.gt.0.001 ) then
        write(qq,'('' ReadTable: The energy mesh of the file read in '',
     1  ''by the TABLE READ command does not agree with this version '',
     2  ''of Cloudy.'')')
        write(qq,'('' ReadTable: Was the file generated by an older '',
     1  ''version of the code?'')')
        write(qq,'('' ReadTable: Number of points read in='',i5)')
     1  nPoints
        write(qq,'('' ReadTable: input, internal energies='',1p,2e12.4)'
     1  )tmn(nPoints) , anu(nPoints) 
        stop
      endif
*
*     make sure rest of array has valid zeros
      do i=nPoints+1,ncell
        ConTabRead(i) = 0.
      end do
*
      close(io)
*
      end
