*
*******************************************************************
      subroutine RebinAtlas(n1,n2)
      implicit none
*rebin the atlas continuum grid onto the cloudy grid
      include "ncell.par"
      include "rfield.com"
      include "corind.com"
      include "widflx.com"
      include "gaunts.com"
      include "StarSize.com"
      include "IOqq.com"
*
*     local variables
      real c,rydb,con1,slope2,power,val,sum,x1,x2,v2,v1,term
      integer i , j , n1,n2,i1,i2
      real edge1,edge2
*     real flux1(ncell),freq1(ncell)
*     flux1 changed to gff, freq1 changed to anu3
*
* I want to insure that the stellar arrays are in increasing frequency order
* here.  the Kurucz files were in wavelength order, so it's necessary to swap
      if(anu2(2).gt.anu2(1)) then
         do j=1,n2
           anu3(j)=anu2(j)
           gff(j)=flux(j)
         end do
      else
*        this is the code for kurucz, n2 is number of energy cells, 1221
         do j=1,n2
           anu3(j)=anu2(n2+1-j)
           gff(j)=flux(n2+1-j)
         end do
      end if
*
      c=299792458.0
      rydb=10967758.30
* These are the speed of light and H-atom Rydberg constant in MKS units, for 
* conversion of frequency to rydberg units.  The infinite mass
* R value is 10973731.534 and the proton to electron mass ratio is
* 1836.152701 for this calculation.
      con1=1./(c*rydb)
*
      do j=1,n2
        anu3(j)=anu3(j)*con1
      end do
*
      slope2=2.0
* this is the Rayleigh-Jeans slope in F_nu, for low frequency extrapolation
*
*     n1 is number of cloudy energy cells
      do j=1,n1
*       lower and upper bounds of this energy cell
        edge1=anu(j)+0.5*widflx(j)
        edge2=anu(j)-0.5*widflx(j)
*       anu3(1) is lower energy bound of kurucz input
*       changed from edge1 to edge2 96 apr 16 gjf due to zero in mid of array
        if (edge2.lt.anu3(1)) then
          ContBoltz(j)=gff(1)*((anu(j)/anu3(1))**slope2)
        else
          if (edge2.gt.anu3(n2)) then
             ContBoltz(j)=0.0
          else
            do i=1,n2-1
              if(anu3(i).le.edge2.and.anu3(i+1).gt.edge2) i2=i
              if(anu3(i).le.edge1.and.anu3(i+1).gt.edge1) i1=i
            end do
* Do the case where the grid point and its edges are between two of the
* stellar model points:  add segments with power-law interpolation up to
* do the averaging.
*
* When edge2 < anu3(1) i2 is equal to 0 and i1 is equal to 1, so do it 
* as if i1 = i2 to avoid going outside array bounds.
              if (i1.eq.i2.or.i2.eq.0) then
                  power=log(gff(i1+1)/gff(i1))/
     c                    log(anu3(i1+1)/anu3(i1))
                  val=power*log(anu(j)/anu3(i1))
                  ContBoltz(j)=gff(i1)*exp(val)
              else
                  sum=0.0
*
                  do i=i2,i1
                  power=log(gff(i+1)/gff(i))/
     c                     log(anu3(i+1)/anu3(i))
                  if (i.eq.i2) then
                      x1=edge2
                      x2=anu3(i+1)
                      v1=gff(i)*exp(power*log(x1/anu3(i)))
                      v2=gff(i+1)
                  end if
                  if (i.eq.i1) then
                      x2=edge1
                      x1=anu3(i)
                      v2=gff(i)*exp(power*log(x1/anu3(i)))
                      v1=gff(i)
                  end if
                  if (i.gt.i2.and.i.lt.i1) then
                      x1=anu3(i)
                      x2=anu3(i+1)
                      v1=gff(i)
                      v2=gff(i+1)
                  end if
                  term=anu3(i)*v1
                  if (abs(power+1.).le.0.001) then
                      val=x1*v1*log(x2/x1)
                  else
                      val=((x2/x1)**(power+1.)-1.)
                      val=val*x1*v1/(power+1.)
                  end if
                  sum=sum+val
                  end do
*
                  ContBoltz(j)=sum/widflx(j)
              end if
          end if
      end if
      end do
*
      end
