*
*******************************************************************
      subroutine RebinRauch(stflux,wlin,flin)
      implicit none
* rebin the rauch continuum grid onto the cloudy grid by K Volk
      integer nvals
      parameter (nvals=19951)
*     Arrays passed in from "GetRauch"
      real stflux(nvals),wlin(nvals),flin(nvals)
*
* This is mostly a copy of "RebinAtlas" except that I pass in the arrays 
* rather than the integer values, since both values needed will 
* be fixed before this routine is called and since the model arrays are 
* much larger than the Cloudy grid size.  Also I do linear interpolation
* rather than log-log interpolation here.
*
* stflux holds the current model flux densities F_nu in erg/s/cm^2/Hz.
* wlin holds the wavelengths in Angstroms.
* flin will have the rebinned fluxes in it upon return.
*
*     limit to number of cells in frequency array
*     NB is this every is changed, also need to change formats in
*     Kevin Volk's stellar atmospheres codes
      include "SpeedLgt.par"
      include "ncell.par"
      include "rfield.com"
      include "StarSize.com"
      include "widflx.com"
      include "IOqq.com"
      real wlryd(nvals)
*
*     local variables
      real rydb,slope(nvals),value,sum,x1,x2,v2,v1,slope2
      integer i , j , i1,i2
      real edge1,edge2
*
      rydb=10967758.30
* These are the speed of light and H-atom Rydberg constant in MKS units, for
* conversion of frequency to rydberg units.  The infinite mass
* R value is 10973731.534 and the proton to electron mass ratio is
* 1836.152701 for this calculation.
*
* Convert wavelengths from angstroms to rydberg; in the calling routine 
* I assured that the wavelengths are decreasing in the array.
*
      do j=1,nvals
        wlryd(j)=1.e+10/(wlin(j)*rydb)
      end do
* Calculate the slopes for interpolation between the Rauch model points.
* Since the model points are closely spaced I will use linear interpolation.
*
      do j=1,nvals-1
        slope(j)=(stflux(j+1)-stflux(j))/(wlryd(j+1)-wlryd(j))
      end do
*
      slope2=2.0
* this is the Rayleigh-Jeans slope in F_nu, for "low" frequency extrapolation.
*
* Actually the extrapolation does not match too well for the lower temperature
* models, but if the long wavelength stellar continuum level matters in the 
* results then something strange is going on anyway.
*
      do j=1,nRauch
*       lower and upper bounds of this energy cell
        edge1=anu(j)+0.5*widflx(j)
        edge2=anu(j)-0.5*widflx(j)
        if (edge2.lt.wlryd(1)) then
          flin(j)=stflux(1)*((anu(j)/wlryd(1))**slope2)
        else
          if (edge2.gt.wlryd(nvals)) then
             flin(j)=0.0
          else
            i1=0
            i2=0
*
            do i=1,nvals-1
              if(wlryd(i).le.edge2.and.wlryd(i+1).gt.edge2) i2=i
              if(wlryd(i).le.edge1.and.wlryd(i+1).gt.edge1) i1=i
            end do
*
            sum=0.
*
            if (i1.eq.i2) then
              flin(j)=stflux(i1)+slope(i1)*(anu(j)-wlryd(i1))
            else
*
              do i=i2,i1
              v1=max(wlryd(i),edge2)
              v2=min(wlryd(i+1),edge1)
              value=v2-v1
              x1=stflux(i)+slope(i)*(v1-wlryd(i))
              x2=stflux(i)+slope(i)*(v2-wlryd(i))
              sum=sum+(x1+x2)*value
              end do
*
              sum=sum/(2.*widflx(j))
              flin(j)=sum
            end if
*
          end if
        end if
      end do
*
      end
