******************************************************************************
      subroutine SetLine
      implicit none
*set up internal parameters and pointers for all transferred lines
      include "nta.par"
      include "ncell.par"
      include "ipLnArry.par"
      include "nwindline.par"
      include "WavNRyd.par"
      include "rfield.com"
      include "pi4.par"
      include "rydlam.par"
      include "t1cm.par"
      include "e1cm.par"
      include "trline.com"
      include "TLineARR.com"
      include "MaxSafe.com"
      include "LineLabl.com"
      include "WindData.com"
      include "KatyaDat.com"
      include "fston.com"
      include "IOqq.com"
*
*     local variables
      integer i,ipLinSafe , ipoint , ip , iWavLen
      real eina,abscf , GetGF
*     degrees kelvin per unit wavenumber
*     real t1cm
*     parameter (t1cm=1.43883)
*     energy ergs per unit wavenumber
*     real e1cm
*     parameter (e1cm=1.98648e-16)
      character chLineLbl*10 , chLab*10 , chIonLbl*4
*
*     this routine sets continuum pointers for all lines
*     which are transferred as heavy element lines
*
*     Verner's FeII lines - do first so following labels will over write this
*     only call if katya data linked in
      if( lgKatyaData .and..not.lgSimKatya ) call PointFe2
*
*     level 2 heavy element lines
      do i=1,NWINDLINE
        WindLine(ipLnRyd,i) = WindLine(ipLnEnrWN,i) * WavNRyd
        WindLine(ipLnIpCont,i) = ipoint( WindLine(ipLnRyd,i) )
*       derive the A, call to function is gf, wl (A), g_up
        WindLine(ipLnAul,i) = eina( WindLine(ipLnGF,i) ,
     1  WindLine(ipLnEnrWN,i) , WindLine(ipLnGu,i) )
        WindLine(ipLnDampRel,i) = WindLine(ipLnAul,i) * 
     1  WindLine(ipLnWlAng,i)*1e-8 / pi4
*       derive the abs coef, call to function is gf, wl (A), g_low
        WindLine(ipLnOpac,i) = abscf( WindLine(ipLnGF,i) ,
     1  WindLine(ipLnEnrWN,i) , WindLine(ipLnGl,i) )
*       excitation energy of transition in degrees kelvin
        WindLine(ipLnBolt,i) = t1cm * WindLine(ipLnEnrWN,i)
*       energy of photon in ergs
        WindLine(ipLnEnrErg,i) = e1cm * WindLine(ipLnEnrWN,i)
        ip = ipoint( WindLine(ipLnRyd,i) )
        WindLine(ipLnIpCont,i) =  ip
        if(chLineLabel(ip).eq.'    ') chLineLabel(ip) =
     1  chIonLbl(WindLine(1,i))
      end do
*666  write(6,'(1x,a10,1p,3e10.2)')
*    1 (chLineLbl( WindLine(1,i) ), WindLine(ipLnWlAng,i),
*    2 anu( int(WindLine(ipLnIpCont,i))), WindLine(ipLnAul,i),
*    3 i=1,NWINDLINE,1000)
*
*     level 1 heavy elements line array
      do i=1,nTauLines
        TauLines(ipLnRyd,i) = TauLines(ipLnEnrWN,i) * WavNRyd
        chLab(1:4) = chIonLbl(TauLines(1,i))
        TauLines(ipLnIpCont,i) = ipLinSafe( TauLines(ipLnRyd,i) ,
     1   chLab(1:4)   )
        if( TauLines(ipLnGF,i ) .gt. 0. ) then
*         the gf value was entered
*         derive the A, call to function is gf, wl (A), g_up
          TauLines(ipLnAul,i) = eina( TauLines(ipLnGF,i) ,
     1    TauLines(ipLnEnrWN,i) , TauLines(ipLnGu,i) )
        else if(TauLines(ipLnAul,i).gt.0. ) then
*         the Einstion A value was entered
*         derive the gf, call to function is A, wl (A), g_up
          TauLines(ipLnGF,i) = GetGF( TauLines(ipLnAul,i) ,
     1    TauLines(ipLnEnrWN,i) , TauLines(ipLnGu,i) )
        else
          write(qq,'('' level 1 line does not have valid gf or A'')')
          write(qq,'('' This is SetLine'')')
          stop
        endif
        TauLines(ipLnDampRel,i) = TauLines(ipLnAul,i) * 
     1  TauLines(ipLnWlAng,i)*1e-8 / pi4
*       derive the abs coef, call to function is gf, wl (A), g_low
        TauLines(ipLnOpac,i) = abscf( TauLines(ipLnGF,i) ,
     1  TauLines(ipLnEnrWN,i) , TauLines(ipLnGl,i) )
*       excitation energy of transition in degrees kelvin
        TauLines(ipLnBolt,i) = t1cm * TauLines(ipLnEnrWN,i)
*       energy of photon in ergs
        TauLines(ipLnEnrErg,i) = e1cm * TauLines(ipLnEnrWN,i)
*begin sanity checks
*       line wavelength must be gt o
        if( TauLines(ipLnWlAng,i).le.0. ) then
          write(qq,'('' SetLine finds insane line wavelength'',
     1    '' <=0, line number'',i4,'' wl='',1p,e10.2)') i,
     2     TauLines(ipLnWlAng,i)
        endif
*end sanity checks
      end do
*     option to print out whole thing with "trace lines" command
      if( lgTrLine ) then
        write(qq,'(1X,''      WL(Ang)   E(RYD)   IP  '',
     1  '' gl  gu      gf       A        damp     abs K'')')
        do i=1,nTauLines
          chLab = chLineLbl(TauLines(1,i))
          iWavLen = TauLines( ipLnWlAng ,i)
          if(iWavLen.gt.1 000 000 ) then
            iWavLen = iWavLen / 10000
          else if(iWavLen.gt.10 000 ) then
            iWavLen = iWavLen / 1000
          endif
          write(qq,'(1X,a10,i5,1P,E10.3,1X,3I4,E10.2,3E10.2)')
     1    chLab, iWavLen,
     1    rydlam/TauLines(ipLnWlAng,i),
     1    int(TauLines(ipLnIpCont,i)),
     2    int(TauLines(ipLnGl,i)),
     2    int(TauLines(ipLnGu,i)),
     2    TauLines(ipLnGF,i),
     2    TauLines(ipLnAul,i),
     3    TauLines(ipLnDampRel,i),
     2    TauLines(ipLnOpac,i)
        end do
        do i=1,NWINDLINE
          chLab = chLineLbl(WindLine(1,i))
          iWavLen = WindLine( ipLnWlAng ,i)
          if(iWavLen.gt.1 000 000 ) then
            iWavLen = iWavLen / 10000
          else if(iWavLen.gt.10 000 ) then
            iWavLen = iWavLen / 1000
          endif
          write(qq,'(1X,a10,i5,1P,E10.3,1X,3I4,E10.2,3E10.2)')
     1    chLab,iWavLen,
     1    rydlam/WindLine(ipLnWlAng,i),
     1    int(WindLine(ipLnIpCont,i)),
     2    int(WindLine(ipLnGl,i)),
     2    int(WindLine(ipLnGu,i)),
     2    WindLine(ipLnGF,i),
     2    WindLine(ipLnAul,i),
     3    WindLine(ipLnDampRel,i),
     2    WindLine(ipLnOpac,i)
        end do
      endif
*
      if( lgTrLine ) then
        write(qq,'('' Largest offset in ipLinSafe was'',i5,
     1  '' at an energy of'',1p,e10.2,'' Ryd.'')') nMaxSafe,EnergySafe
      endif
*
*     this is an option to kill fine structure line optical depths
      if( .not.lgFstOn ) then
        do i=1,nTauLines
          if( TauLines(ipLnEnrWN,i).lt. 10 000. ) then
            TauLines(ipLnOpac,i) = 0.
          endif
        end do
      endif
*
      end
