******************************************************************************
      subroutine SumHeat
      implicit none
*evaluate all heating agents to determine total heating for this zone
      include "limelm.par"
      include "itrdim.par"
      include "ncolnt.par"
      include "heat.com"
      include "dcool.com"
      include "heatbg.com"
      include "TePowers.com"
      include "trace.com"
      include "heating.com"
      include "tcool.com"
      include "coolnt.com"
      include "HColRec.com"
      include "phycon.com"
      include "HLineTot.com"
      include "ZoneCnt.com"
      include "NumDeriv.com"
      include "ffheat.com"
      include "holod.com"
      include "IOqq.com"
*
*     local variables
      real trigger , SumCTHeat
      parameter (trigger=0.02)
      real save(20) 
      integer ipsave(20),jpsave(20)
*
      integer i , j , ipnt
      real TotalHeat , oldfac , oldheat , oldtemp,deriv
      integer nzSave , nhit
      save nzSave , oldheat , oldtemp , nhit
*
*     routine to sum up total heating, and print agents if needed
*     it also computes the true deriv, dH/dT
*
*     find heating due to charge transfer
      heating(1,25) = SumCTHeat( )
*
*     get total heating
      TotalHeat = 0.
      do i=1,limelm
        do j=1,limelm
          TotalHeat = TotalHeat + heating(i,j)
        end do
      end do
      htot = TotalHeat 
*
*     check whether Lya heating is important
*     if so then only very small temp changes can be made
      if( heating(1,11)/htot.gt. 0.15 ) then
        lgHCoolng = .true.
      else
        lgHCoolng = .false.
      endif
*
*     add on line heating to this array, heatl was evaluated in sumcool
*     not zero, because of roundoff error
      if( heatl/ctot .lt.-1e-15 ) then
        write(qq,'('' SumHeat gets negative line heating,'',1p,2e10.2,
     1  '' this is insane.'')') heatl,ctot
        write(qq,'('' this is zone'',i4)') nzone
        call ShowMe
        stop
      endif
*
*     now get deriv
      dHTotDT = 0.
*     photoionization of all elements
      do i=1,limelm
*     >>chng 97 mar 18, counter had gone to i-1
*       do j=1,i-1
        do j=1,i
          dHTotDT = dHTotDT + heating(i,j)
        end do
      end do
*     this is deriv of photo heating - 
*     >>chng 96 jan 14
*>>>>>NB heating going down with iincreasing temp is negative
      dHTotDT = -0.7 *dHTotDT/ te
*
*     oldfac was factor used in old implimentation
*     any term using it should be rethought
      oldfac = -0.7 / te
*     carbon monoxide
      dHTotDT = dHTotDT + heating(1,10) * oldfac
*     ly alpha collisoinal heating
      dHTotDT = dHTotDT + heating(1,11) * oldfac
*     line heating
      dHTotDT = dHTotDT + heating(1,23) * oldfac
*     free free heating, propto T^-0.5
*     >>chng 96 oct 30, from heating(1,12) to FreeFreeHeat,
*     do cooling separately assume FreeFreeHeat goes as T^-3/2
*     dHTotDT = dHTotDT + heating(1,12) * (-0.5/te)
      dHTotDT = dHTotDT + FreeFreeHeat * (-1.5/te)
*     grain heating by photo
*     >>chng 97 july 2, change deriv in following, better fit to what came out
      dHTotDT = dHTotDT + heating(1,14) * 0.1 / te
*     grain heating by collsional
      dHTotDT = dHTotDT + heating(1,15) * oldfac
*     helium triplets heating
      dHTotDT = dHTotDT + heating(2,3) * oldfac
*     hydrogen molecule dissociation
      dHTotDT = dHTotDT + heating(1,18) * oldfac
*     >>chng 96 nov 15, had been oldfac below, wrong sign
*     H- heating, goes up with temp since rad assoc does
      dHTotDT = dHTotDT + heating(1,16) * 0.7/te
*     H2+ heating
      dHTotDT = dHTotDT + heating(1,17) * oldfac
*     Balmer continuum and all other excited states
*     - T goes up so does pop and heating
*     but this all screwed up by change in eden
      dHTotDT = dHTotDT + heating(1,2) * oldfac
*     all three body heating, opposite of coll ion cooling
      dHTotDT = dHTotDT + heating(1,4) * oldfac
*     bound electron recoil heating
      dHTotDT = dHTotDT + heating(1,19) * oldfac
*     compton heating
      dHTotDT = dHTotDT + heating(1,20) * oldfac
*     extra heating source, usually zero
      dHTotDT = dHTotDT + heating(1,21) * oldfac
*     pair production
      dHTotDT = dHTotDT + heating(1,22) * oldfac
*     deriv of heating due to collisions of H lines, heating(1,24)
      dHTotDT = dHTotDT + dHLTot
*     heating due to Katya's FeII atom
      if( heating(26,28).gt.0. ) then
        dHTotDT = dHTotDT + dfcool
      endif
*
*     possibility of getting emperical heating derivative
*     normally false, set true with 'set numerical derivatives' command
      if( lgNumDeriv ) then
        if( nzone.gt.2 .and. nzone.eq.nzSave .and. oldtemp.ne.te
     1  .and.nhit.gt.4) then
*         hnit is number of trys on this zone - use to stop numerical problems
*         do not evaluate numerical deriv until well into solution
          deriv = (oldheat - htot)/(oldtemp-te)
          dHTotDT = deriv
        else
          deriv = dHTotDT
        endif
*       this happens when new zone starts
        if(nzone.ne.nzSave) nhit = 0
        nzSave = nzone
        nhit = nhit + 1
        oldheat = htot
        oldtemp = te
      endif
*
      if( lgTrace.and.lgHeatBug ) then
        ipnt = 0
        do i=1,limelm
          do j=1,limelm
            if( heating(i,j)/htot.gt.trigger ) then
              ipnt = min( 20 , ipnt + 1 )
              ipsave(ipnt) = i
              jpsave(ipnt) = j
              save(ipnt) = heating(i,j) / htot
            endif
          end do
        end do
*
*       now check for possible line heating not counted in 1,23
*       there should not be any significant heating source heree
*       since they would not be counted in deriv correctly
        do i=1,ncltot
          if( heatnt(i)/htot.gt.trigger) then
            ipnt = min( 20 , ipnt + 1 )
            ipsave(ipnt) = -1
            jpsave(ipnt) = i
            save(ipnt) = heatnt(i) / htot
          endif
        end do
*
*       write(qq,'('' heat fe2'',1p,8e10.2)')fcool,dfcool,heating(26,28)
        write(qq,'(5x,''SumHeat HTOT'',1P,E10.3,'' Te:'',e10.3,
     1 '' dH/dT'',e9.2,2X,0P,6(2I3,F6.3))')
     1  htot,te,dHTotDT,(ipsave(i),jpsave(i),save(i),i=1,min(6,ipnt))
        if( ipnt.gt.6 ) then
          write(qq,'(31x,7(2i4,f6.3))') (ipsave(i),jpsave(i),save(i),
     1    i=7,ipnt)
        endif
      endif
*
      end
