*
******************************************************************************
      subroutine werne( nstar , temp , alogg , nspec )
      implicit none
*read in and interpolate on Werner grid of PN atmospheres, by K Volk
      real temp, alogg,dev1,dev2,term
      integer nmods,nstar,nspec,j,i,ierr,istop1,jmod
      include "ncell.par"
      include "limspc.par"
      include "terp.com"
      include "rfield.com"
      include "path.com"
      character chLine*80
      include "called.com"
      include "StarSize.com"
      include "pthset.com"
      include "NumUpper.com"
      include "IOqq.com"
*     parameter (ndata=865,nmods=20)
      parameter (nmods=20)
* SUBROUTINE "WERNE" WAS ADDED (28 DEC 1992) TO READ FROM THE SET OF
* HOT WHITE DWARF MODEL ATMOSPHERES FROM KLAUS WERNER AT KEIL.  THE 
* VALUES ARE READ IN (ENERGY IN RYDBERG UNITS, LOG10(F_NU) [F_NU IN CGS 
* UNITS]) FOR ANY OF 20 MODELS.  T_EFF VALUES OF 80, 100, 120, 140, 160,
* 180 AND 200 THOUSAND K AND LOG(G) VALUES OF 5.0, 6.0, 7.0, AND 8.0 ARE
* AVAILABLE.  THIS VERSION DOES INTERPOLATION.  IF A MODEL HAS A T_EFF
* AND A G VALUE WITHIN 1% OF THE VALUES SPECIFIED IN THE PARAMETERS, THAT
* MODEL IS CHOSEN FOR USE.  EACH MODEL HAS 514 POINTS, THE LONGEST WAVELENGTH
* POINT BEING AN EXTRAPOLATION FROM THE NEXT TWO POINTS.  THE FILE IS A
* FORMATTED DIRECT ACCESS FILE WITH RECORD LENGTH 7*865=6055 BYTES, FORMAT 
* 865I7=3*250i7,115i7.
*
* FILE USED: WERNER.MOD (CONTAINS THE WAVELENGTH ARRAY, THEN THE MODELS)
*
* TESTS INDICATE THAT THE INTERPOLATION AND RESCALING PRESERVE THE SET
* T_EFF VALUE (MEASURED BY THE SURFACE FLUX) TO WITHIN ABOUT 100 K FOR
* ANY TEMPERATURE IN THE 80,000 TO 200,000 K RANGE.  RESCALING BY FACTORS
* OF ORDER 1.01 TO 1.1 ARE NEEDED TO ACHIEVE THIS, SINCE THE INTERPOLATION BY
* ITSELF CAN PRODUCE SOME DEVIATIONS.  THE LARGEST RESCALING IS NEEDED AT 
* "LOW" T_EFF AND LOG(G) VALUES.  FOR HIGH TEMPERATURES THE SCALING IS SELDOM
* LARGER THAN 1.01.
*
* THESE RESCALING RESULTS ARE FOR THE FULL MODELS, NOT THE REBINNED VERSIONS.
* THE NUMBER OF GRID POINTS IS NOT CHANGED TOO MUCH, SO THE RESULTS SHOULD
* NOT BE AFFECTED TOO MUCH.
*
*
*     nmods is the number of accessable models in the werner model set.
*     data set 1 is the wavelength grid, so there are nmods+1 data sets
*     in the main file.  each set has 865 re-binned values, increasing in 
*     frequency.  the original models had 514 values.
*
      real wl(ncell), teff(nmods),xlogg(nmods)
      integer ivals(ncell),modnum(7,4)
      real tval(7),gval(4)
      logical lgAgain
*
      data tval/80000.,100000.,120000.,140000.,160000.,180000.,
     1          200000./
      Data gval/5.,6.,7.,8./
      data modnum/2,6,10,13,16,18,20,3,7,10,13,16,18,20,4,8,11,14,
     1            16,18,20,5,9,12,15,17,19,21/
      data teff/80000.,80000.,80000.,80000.,100000.,100000.,100000.,
     1          100000.,120000.,120000.,120000.,140000.,140000.,
     1          140000.,160000.,160000.,180000.,180000.,200000.,
     1          200000./
      data xlogg/5.,6.,7.,8.,5.,6.,7.,8.,6.,7.,8.,6.,7.,8.,7.,8.,
     1           7.,8.,7.,8./
*
*     add together path to form atlas file 
*
*     true is special path set, false if file in same dir
      if( lgPathSet ) then
        j = 1
        lgAgain = .true.
        i = 1
*     >>chng 97 may 13, get rid of statement number
*       do i=1,80
        do while( lgAgain.and.i.le.80 )
          if( chPath(i:i).ne.' ') then
            chLine(j:j) = chPath(i:i)
            j = j + 1
          else
            lgAgain = .false.
          endif
          i = i + 1
        end do
*2      continue
*       j now points to first free character
        chLine(j:j+9) = 'werner.mod'
*       end string with zero character
        chLine(j+10:j+10) = char(0)
      else
        chLine(1:10) = 'werner.mod'
*       end string with zero character
        chLine(11:11) = CHAR(0)
      endif
*
      ierr=1
      istop1=1
      open(unit=93,file=chLine,status='OLD',iostat=ierr,
     1     access='DIRECT',form='FORMATTED',recl=7*ncell)
*
*     something went wrong
      if( ierr.ne.0 ) then
          write(qq,'(''ERROR: The Klaus Werner stellar atmosphere '',
     1    ''file was not found.''/''Sorry.'')')
          stop
      end if
*
      read(93,'(4000i7)' ,iostat=ierr ,rec=1) (ivals(j),j=1,nWerner)
*     following must change if GetWerner changes
*999  format(4000i7)
      if( ierr.ne.0 ) then
        write(qq,'('' error getting Werner wavelength scale.'')')
        stop
      endif
*
      do j=1,nWerner
        wl(j)=10.**(5.-float(ivals(j))/1000000.)
      end do
*
      jmod=-1
*
*
*     TEMP and ALOGG are desired temp and log g
*
*     this may (or may not) catch bad input parameters
      if( temp.lt.80 000. .or. temp.gt.200 000. ) then
        write(qq,'('' Entered temperature is outside range of grid.'')')
        stop
      else if( alogg.lt.5. .or. alogg.gt.8. ) then
        write(qq,'('' Entered gravity is outside range of grid.'')')
        stop
      endif
*
      lgAgain = .true.
      j = 1
*     >>chng 97 may 13, get rid of statement nmber
*     do j=1,nmods
      do while(lgAgain .and. j.le.nmods )
        dev1=abs(temp/teff(j)-1.)
        dev2=abs(alogg-xlogg(j))
        if (dev1.le.0.01.and.dev2.le.0.00414) then
          jmod=j+1
*         go to730
          lgAgain = .false.
        endif
        j = j + 1
      end do
*
*730  continue
      if(jmod.eq.-1) then
        call getmod(temp,alogg,modnum,tval,gval,ierr,wl)
        if (ierr.gt.0) then
          write(qq,'('' Error occurred in getmod.'')')
          stop
        endif
      else
        read(93,'(4000i7)' ,iostat=ierr,rec=jmod) (ivals(j),j=1,nWerner)
        if( ierr.ne.0 ) then
          write(qq,'('' Error trying to read Werner atmosphere'',i4)')
     1    jmod
          stop
        endif
*
        do j=1,nWerner
          flux(j)=3.-float(ivals(j))/100000.
        end do
*
        if(lgTalk)write(qq,'(23X,''* <<< K.Volk s  K. Werner model ''
     1  ,i2,'' read in.  T_eff = '',F8.1,''  log(g) = '',F6.3,
     2  ''>>> *'')') jmod-1,teff(jmod-1),xlogg(jmod-1)
      endif
*
      do j=1,nWerner
*       gjf TSLOP(NSPEC,J)=FLUX(j)
*       TSLOP is f_nu for each TNU point
        tslop(nspec,j) = 10.**flux(j)
        tnu(nspec,j)=wl(j)
        if( j.gt.1.and.j.lt.nWerner) then
          if( tnu(nspec,j).eq.tnu(nspec,j-1)) then
            if(tnu(nspec,j).lt.10.) then
              term=1.0
            else
              term=10.0
            endif
            tnu(nspec,j)=tnu(nspec,j)+5.e-07*term
            tnu(nspec,j-1)=tnu(nspec,j-1)-5.e-07*term
*           this is to handle jumps at ionization or level absorption edges
*           by spacing them 1.0e-06 rydberg apart -- more or less, as this is
*           at the edge of the real variable precision.  For values above 10.0
*           the jump is made 10 times larger to allow it to show up.
          endif
        endif
      end do
      do j=nWerner+1,ncell
        tslop(nspec,j) = 0.
      enddo
*
      nstar = nWerner
      istop1=0
      close(unit=93)
*
      end
