******************************************************************************
      subroutine zonsrt( chMode )
      implicit none
*set variables that change with each zone, like radius, depth
      character chMode*4
*
      include "itrdim.par"
      include "nzlim.par"
      include "dRdVeff.com"
      include "struc.com"
      include "sphere.com"
      include "radius.com"
      include "filfac.com"
      include "cover.com"
      include "drnxt.com"
      include "r1r0sq.com"
      include "cylind.com"
      include "globul.com"
      include "dr2sml.com"
      include "out2in.com"
      include "drsign.com"
      include "cpres.com"
      include "phycon.com"
      include "ZoneCnt.com"
      include "tpred.com"
      include "VolRef.com"
      include "IOqq.com"
*
*     local variables
      real ratio , vin,vout,rm , dt1 , dt2 , DTaver,outwrd
      integer ioff , i
      parameter (ioff=5)
      logical lgOscil
      save dt2 , DTaver
*
*     called at start of a zone to update all variables
*     having to do with starting this zone
*
*     this sub can be called in two ways, 'incr' means increment
*     radius variables, while 'init' means only initialize rest
*
      if( chMode.eq.'incr' ) then
*       update radius variables
        drad = drNext
        depth = depth + drad
        radius = radius + drad * dRadSign
*       following is distance to globule, usually doesnt matter
        glbdst = glbdst - drad
*       if gt 5th zone, and not constant pressure, and not oscillating te
*       option to guess next temperature : option to predict next temperature
*       nzlim is dim of structure variables saving temp, do data if nzone>nzlim
        if( chCPres.ne.'CPRE' .and. nzone.gt.ioff+1 .and.
     1    nzone.lt.nzlim)then
          TeInit = te
          lgOscil = .false.
          dt1 = 0.
          dt2 = 0.
          DTaver = 0.
          do i=nzone-ioff,nzone-1
            dt1 = dt2
            dt2 = testr(i-1) - testr(i)
            DTaver = DTaver + dt2
            if(dt1*dt2.lt.0. ) lgOscil = .true.
          end do
          if(.not.lgOscil ) then
            DTaver = DTaver / float(ioff)
*           don't want to overcorrect, use smaller of two
            DTaver = sign( min(abs(DTaver),abs(dt2)) , DTaver)
*           do not let it change by more than 5% of current temp
            DTaver = sign( min(abs(DTaver),0.05*te) , DTaver)
            te = te - DTaver
          else
*           temp was oscillating - too dangerous to do anything
            DTaver = 0.
          endif
          TeProp = te
        endif
      else if( chMode.eq.'init' ) then
*       initialize the variables at start of calculation
        depth = 1d-30
        radius = rinner
        drNext = drad
*       depth variable for globule
        glbdst = glbrad
*       following is for out to in geometry
        if( lgOut2In ) then
          dRadSign = -1.
        else
*         this is the normal case
          dRadSign = 1.
        endif
      else
        write(qq,'('' ZONSRT called with insane argument, '',A4)')chMode
        call insane
        stop
      endif
*
*     check whether zone thickness is too small relative to radius
      if( chCPres.eq.'GLOB' ) then
        ratio = drad / (glbdst+drad)
*       this only matters for globule command
        if( ratio.lt.1e-14 ) then
          lgdR2Small = .true.
        else
          lgdR2Small = .false.
        endif
      endif
*
      FillFac = min( 1. , fiscal * real(radius/rinner)**filpow )
*
*     effective radius
      dReff = drad * FillFac
*     area factor
      r1r0sq = ( (radius-drad*dRadSign/2.) /rinner) ** 2
*
*     following only approximate, used for center of next zone
      dRNeff = drNext * FillFac
*
*     RADIUS is outer radius of this zone
*     RINNER is inner radius of nebula; dVeff is the effective vol element
*     dVeff is vol rel to inner radius, so has units cm
*     for plane parallel dVeff is dReff
      if( drad/radius.gt.0.01 ) then
        vin =  ((radius-drad)/rinner) * FillFac *(min(
     1  real(radius-drad),
     1  CylindHigh)/real(rinner))*(radius-drad)
        vout = (radius/rinner) * FillFac *(min(real(radius),CylindHigh)/
     1  rinner)* radius
*       just the difference in the two vols
        dVeff = ( vout - vin ) / 3.
      else
*       RM is the middle of the zone;
*       radius is always the OUTER edge of the current zone
        rm = radius - drad/2.
        dVeff =  (rm/rinner) * drad* FillFac * (min(rm,CylindHigh)/
     1  rinner)
      endif
*
*     covering factor, default if unity
      dVeff = dVeff * covgeo
*
*     these are needed for line intensities in outward bean
*     lgSphere set, covrt usually 1, not lgSphere then 0
*     outwrd is 1/2 when lgSphere not set, 1 when set
      outwrd = (1.+covrt)/2.
*     this includes both the effective vol, and 1/r^2 dilution
      OutFac = outwrd * radius * dReff / (radius + 2.*drad)
      RefFac = (1.-outwrd) * dReff * r1r0sq
      if( lgSphere ) then
*       inward beams do not go in when lgSphere set since geometry symmetric
        BeamInIn = 0.
        BeamInOut = radius * dReff / (radius + 2.*drad)
      else
        BeamInIn = dReff * r1r0sq
*       inward beams do not go out
        BeamInOut = 0.
      endif
*     factor for outwardly directed part of line
      BeamOutOut = radius * dReff / (radius + 2.*drad)
*
      end
